import javax.swing.*;
import java.awt.*;
import java.io.*;
import java.util.*;
import java.awt.event.*;
import javax.swing.table.*;

public class TuringMachine extends JFrame {

  public TuringMachine()
  {
    super("Turing Machine Simulator");
    setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    init();
  }

  public static void main(String[] args) {
    TuringMachine sim = new TuringMachine();
    sim.setSize(600,600);
    sim.setVisible(true);
  }

    /** Initializes the applet TuringMachine */
    public void init() {
	Fields.tape = new Tape();
	Fields.currentDirectory = System.getProperty("user.dir");
        initTape();
        initComponents();
        showGreeting();
        readRuleLists();
    }
    
    
    /** Writes a greeting to the messages window */
    private void showGreeting() {
        showMessage("Welcome!");
        showMessage("This is a Turing Machine Simulator Applet.");
        showMessage("Select a rule list, enter an input and then click on the start button!");
    }
    
    
    /** Writes Strings as messages to the messages output window */
    private void showMessage(String message) {
        textAreaMessages.append(message);
        textAreaMessages.append("\n");
        textAreaMessages.scrollRectToVisible(new Rectangle(0, (int) (textAreaMessages.getSize().getHeight() - 10), (int) (textAreaMessages.getSize().getWidth()), 10));
    }
    
    
    /** Initializes the tape with the specified input */
    public void initTape() {
	Fields.tape.clearTape();
       	for (int i = -(Fields.fieldNum); i < 0; i++) {
	    Fields.tape.insertRightEnd(new NodeField(Fields.symbolEmptyField, i));
	}
        NodeField nf = new NodeField(Fields.input.charAt(0), 0);
        Fields.tape.insertRightEnd(nf);
        Fields.tape.current = Fields.tape.rightEnd;
	for (int i = 1; i < Fields.input.length(); i++) {
	    Fields.tape.insertRightEnd(new NodeField(Fields.input.charAt(i), i));
	}
	for (int i = Fields.input.length(); i < (Fields.input.length() + Fields.fieldNum); i++) {
	    Fields.tape.insertRightEnd(new NodeField(Fields.symbolEmptyField, i));
	}
        System.out.println(Fields.tape);
    }
    
    
    /** Performs a single step */
    public void doSingleStep() {
        if (Fields.currentState == Fields.finalState) {
            if ((Fields.timer != null) && (Fields.timer.isRunning())) {
                Fields.timer.stop();
            }
            Fields.tmRunMode = 3;
            showMessage("\nTuring Machine reached final state.");
            showMessage("==> Turing Machine accepts input.\n");
            return;
        }
        Rule r = Fields.ruleList[Fields.selection].getRule(Fields.currentState,
            ((NodeField) Fields.tape.current.getContent()).getSymbol());
        if (r != null) {
            int index = Fields.ruleList[Fields.selection].indexOf(r);
            if (Fields.showCurrentRule) {
                ListSelectionModel sm = tableRuleList.getSelectionModel();
                sm.setSelectionInterval(index, index);
                tableRuleList.scrollRectToVisible(new Rectangle(0, (index * tableRuleList.getRowHeight()), tableRuleList.getWidth(), tableRuleList.getRowHeight()));
            }
            labelStatsSteps.setText("Steps: " + (++Fields.stepCounter));
            panelStats.repaint();
            ((NodeField) Fields.tape.current.getContent()).setSymbol(r.getWrite());
            panelTuringMachine.repaint();
            if (r.getMove() == 'l') { 
                ((TuringMachinePanel) panelTuringMachine).moveLeft();
            }
            if (r.getMove() == 'r') { 
                ((TuringMachinePanel) panelTuringMachine).moveRight();
            }
            Fields.currentState = r.getNextState();
        } else {
            if (Fields.timer != null) {
		Fields.timer.stop();
	    }
            showMessage("ERROR: Could not find rule for state " + Fields.currentState + " and symbol " + ((NodeField) Fields.tape.current.getContent()).getSymbol());
            return;
        }

    }
    
    
    /** Returns the rule list that is saved in the current file */
    public void readRuleLists() {

	/* variables */
	int state;
	char read;
	char write;
	char move;
	int nextState;
        File file;
	Rule rule;
	String line;
	BufferedReader br;
	FileInputStream in;
	StringTokenizer stringTokenizer;
        Fields.ruleList = new RuleList[Fields.ruleListNames.length];
        
	/* Reads out the file */        
        for (int i = 0; i < Fields.ruleListNames.length; i++) {
            Fields.ruleList[i] = new RuleList();
            try {

		String filename = Fields.ruleListNames[i];
		file = new File(filename == "file" ?
				Fields.selectedFile :
				Fields.ruleListNames[i]);

		System.err.println("Trying to read rule list from file " + file.toString());

                in = new FileInputStream(file);
                br = new BufferedReader(new InputStreamReader(in));
	    
                while ((line = br.readLine()) != null) {

		    System.err.println(line);

                    if ((!line.equals(""))
                        && (line.charAt(0) != Fields.commentChar)
                        && (line.length() >= Fields.tokenNumber)) {
                        stringTokenizer = new StringTokenizer(line, Fields.delimiter);
                        if (stringTokenizer.countTokens() != Fields.tokenNumber) {
                            showMessage("Error: Wrong number of tokens in file!");
                        } else {
                            state = Integer.parseInt(stringTokenizer.nextToken());
                            read = stringTokenizer.nextToken().charAt(0);
                            write = stringTokenizer.nextToken().charAt(0);
                            move = stringTokenizer.nextToken().charAt(0);
                            nextState = Integer.parseInt(stringTokenizer.nextToken());
                            rule = new Rule(state, read, write, move, nextState);
                            Fields.ruleList[i].addRule(rule);
                        }
                    }
                }
            }
            catch (FileNotFoundException fnfe) {
                showMessage("Error: File " + Fields.ruleListNames[i] + " not found!");
            }
            catch (IOException ioe) {
                showMessage("Error: " + ioe);
            }
        }
    }
    
    
    /** Saves the specified rule list to the current file */
    public void writeRuleList(String fileName, RuleList ruleList) {

	/* variables */
        File file = new File(fileName);
	Rule rule;
	String line = "";
	PrintStream ps;
	
	/* writing the data to the file */
	try {
            ps = new PrintStream(new FileOutputStream(file));
	    ps.println(Fields.commentChar + file.getName());
	    Enumeration e = ruleList.elements();
	    while (e.hasMoreElements()) {
		rule = (Rule) e.nextElement();
		ps.print(rule.getState());
		ps.print(Fields.delimiter);
		ps.print(rule.getRead());
		ps.print(Fields.delimiter);
		ps.print(rule.getWrite());
		ps.print(Fields.delimiter);
		ps.print(rule.getMove());
		ps.print(Fields.delimiter);
		ps.println(rule.getNextState());
	    }
	}
	catch (FileNotFoundException fnfe) {
	    showMessage("Error: File " + file.getName() + " not found!");
	}
	catch (IOException ioe) {
	    showMessage("Error: " + ioe);
	}
    }
    
    
    /* Creates the table with the data of the specified rule list */
    public void createTable() {
        Enumeration e = Fields.ruleList[Fields.selection = comboBoxRuleList.getSelectedIndex()].elements();
        tableRuleList.setModel(new javax.swing.table.DefaultTableModel (
        new Object [][] {
            
        },
        new String [] {
            "State", "Read", "Write", "Move", "Next State"
        }
        ) {
            Class[] types = new Class [] {
                java.lang.Integer.class, java.lang.Character.class, java.lang.Character.class, java.lang.Character.class, java.lang.Integer.class
            };
            
            public Class getColumnClass (int columnIndex) {
                return types [columnIndex];
            }
        });
        while (e.hasMoreElements()) {
            Rule rule = (Rule) e.nextElement();
            Object[] row = new Object[] {
                new Integer(rule.getState()),
                new Character(rule.getRead()),
                new Character(rule.getWrite()),
                new Character(rule.getMove()),
                new Integer(rule.getNextState())
            };
            ((DefaultTableModel) tableRuleList.getModel()).addRow(row);
        }
    }    
    
    
    /** This method is called from within the init() method to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        MenuBar = new javax.swing.JMenuBar();
        File = new javax.swing.JMenu();
        Options = new javax.swing.JMenu();
        panelTuringMachine = new TuringMachinePanel(600, 100);
        panelControl = new javax.swing.JPanel();
        buttonStart = new javax.swing.JButton();
        buttonPause = new javax.swing.JButton();
        buttonReset = new javax.swing.JButton();
        buttonStep = new javax.swing.JButton();
        panelRuleList = new javax.swing.JPanel();
        panelRuleListSelect = new javax.swing.JPanel();
        comboBoxRuleList = new javax.swing.JComboBox(Fields.ruleListNames);
        buttonRuleListLoad = new javax.swing.JButton();
        scrollPanelRuleList = new javax.swing.JScrollPane();
        tableRuleList = new javax.swing.JTable();
        panelInput = new javax.swing.JPanel();
        textFieldInput = new javax.swing.JTextField();
        buttonInputLoad = new javax.swing.JButton();
        labelStateInitial = new javax.swing.JLabel();
        textFieldStateInitial = new javax.swing.JTextField();
        labelStateFinal = new javax.swing.JLabel();
        textFieldStateFinal = new javax.swing.JTextField();
        panelOptions = new javax.swing.JPanel();
        panelSpeed = new javax.swing.JPanel();
        labelSpeed = new javax.swing.JLabel();
        sliderSpeed = new javax.swing.JSlider();
        Fields.sliderLabelTable = new java.util.Hashtable();
        Fields.sliderLabelTable.put(new Integer(1), new javax.swing.JLabel("Slow"));
        Fields.sliderLabelTable.put(new Integer(1000), new javax.swing.JLabel("Fast"));
        sliderSpeed.setLabelTable(Fields.sliderLabelTable);
        panelShowRule = new javax.swing.JPanel();
        checkBoxShowCurrentRule = new javax.swing.JCheckBox();
        panelStats = new javax.swing.JPanel();
        labelStatsSteps = new javax.swing.JLabel();
        panelMessages = new javax.swing.JPanel();
        scrollPaneMessages = new javax.swing.JScrollPane();
        textAreaMessages = new javax.swing.JTextArea();
        
        File.setText("Menu");
        MenuBar.add(File);
        
        Options.setText("Menu");
        MenuBar.add(Options);
        getContentPane().setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gridBagConstraints1;
        
        panelTuringMachine.setBorder(new javax.swing.border.TitledBorder(
        new javax.swing.border.EtchedBorder(), "Turing Machine", 2, 2));
        panelTuringMachine.setPreferredSize(new java.awt.Dimension(600, 100));
        panelTuringMachine.setMinimumSize(new java.awt.Dimension(600, 100));
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 0;
        gridBagConstraints1.gridwidth = 24;
        gridBagConstraints1.gridheight = 4;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
        getContentPane().add(panelTuringMachine, gridBagConstraints1);
        
        
        panelControl.setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gridBagConstraints2;
        panelControl.setBorder(new javax.swing.border.TitledBorder("Control"));
        panelControl.setPreferredSize(new java.awt.Dimension(400, 75));
        panelControl.setMinimumSize(new java.awt.Dimension(400, 75));
        
        buttonStart.setToolTipText("Start");
        buttonStart.setText("Start");
        buttonStart.setPreferredSize(new java.awt.Dimension(75, 25));
        buttonStart.setMinimumSize(new java.awt.Dimension(75, 25));
        buttonStart.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                buttonStartActionPerformed(evt);
            }
        }
        );
        gridBagConstraints2 = new java.awt.GridBagConstraints();
        gridBagConstraints2.gridwidth = 6;
        gridBagConstraints2.gridheight = 2;
        gridBagConstraints2.fill = java.awt.GridBagConstraints.BOTH;
        panelControl.add(buttonStart, gridBagConstraints2);
        
        
        buttonPause.setToolTipText("Pause");
        buttonPause.setText("Pause");
        buttonPause.setPreferredSize(new java.awt.Dimension(75, 25));
        buttonPause.setMinimumSize(new java.awt.Dimension(75, 25));
        buttonPause.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                buttonPauseActionPerformed(evt);
            }
        }
        );
        gridBagConstraints2 = new java.awt.GridBagConstraints();
        gridBagConstraints2.gridwidth = 6;
        gridBagConstraints2.gridheight = 2;
        gridBagConstraints2.fill = java.awt.GridBagConstraints.BOTH;
        panelControl.add(buttonPause, gridBagConstraints2);
        
        
        buttonReset.setToolTipText("Reset");
        buttonReset.setText("Reset");
        buttonReset.setPreferredSize(new java.awt.Dimension(75, 25));
        buttonReset.setMinimumSize(new java.awt.Dimension(75, 25));
        buttonReset.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                buttonResetActionPerformed(evt);
            }
        }
        );
        gridBagConstraints2 = new java.awt.GridBagConstraints();
        gridBagConstraints2.gridwidth = 6;
        gridBagConstraints2.gridheight = 2;
        gridBagConstraints2.fill = java.awt.GridBagConstraints.BOTH;
        panelControl.add(buttonReset, gridBagConstraints2);
        
        
        buttonStep.setToolTipText("Step");
        buttonStep.setText("Step");
        buttonStep.setPreferredSize(new java.awt.Dimension(75, 25));
        buttonStep.setMinimumSize(new java.awt.Dimension(75, 25));
        buttonStep.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                buttonStepActionPerformed(evt);
            }
        }
        );
        gridBagConstraints2 = new java.awt.GridBagConstraints();
        gridBagConstraints2.gridwidth = 6;
        gridBagConstraints2.gridheight = 2;
        gridBagConstraints2.fill = java.awt.GridBagConstraints.BOTH;
        panelControl.add(buttonStep, gridBagConstraints2);
        
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 4;
        gridBagConstraints1.gridwidth = 16;
        gridBagConstraints1.gridheight = 2;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
        getContentPane().add(panelControl, gridBagConstraints1);
        
        
        panelRuleList.setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gridBagConstraints3;
        panelRuleList.setBorder(new javax.swing.border.TitledBorder("Rule List"));
        panelRuleList.setPreferredSize(new java.awt.Dimension(400, 250));
        panelRuleList.setMinimumSize(new java.awt.Dimension(400, 250));
        
        panelRuleListSelect.setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gridBagConstraints4;
        panelRuleListSelect.setPreferredSize(new java.awt.Dimension(380, 50));
        panelRuleListSelect.setMinimumSize(new java.awt.Dimension(380, 23));
        
        comboBoxRuleList.setSelectedIndex(0);
        comboBoxRuleList.setPreferredSize(new java.awt.Dimension(300, 23));
        comboBoxRuleList.setMinimumSize(new java.awt.Dimension(126, 23));
        comboBoxRuleList.setMaximumSize(new java.awt.Dimension(32767, 32767));
        comboBoxRuleList.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                comboBoxRuleListActionPerformed(evt);
            }
        }
        );
        comboBoxRuleList.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                comboBoxRuleListItemStateChanged(evt);
            }
        }
        );
        gridBagConstraints4 = new java.awt.GridBagConstraints();
        gridBagConstraints4.gridx = 0;
        gridBagConstraints4.fill = java.awt.GridBagConstraints.BOTH;
        panelRuleListSelect.add(comboBoxRuleList, gridBagConstraints4);
        
        
        buttonRuleListLoad.setText("Load");
        buttonRuleListLoad.setPreferredSize(new java.awt.Dimension(75, 25));
        buttonRuleListLoad.setMinimumSize(new java.awt.Dimension(75, 25));
        buttonRuleListLoad.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                buttonRuleListLoadActionPerformed(evt);
            }
        }
        );
        gridBagConstraints4 = new java.awt.GridBagConstraints();
        gridBagConstraints4.fill = java.awt.GridBagConstraints.BOTH;
        panelRuleListSelect.add(buttonRuleListLoad, gridBagConstraints4);
        
        gridBagConstraints3 = new java.awt.GridBagConstraints();
        gridBagConstraints3.gridx = 0;
        gridBagConstraints3.gridwidth = 12;
        gridBagConstraints3.gridheight = 2;
        gridBagConstraints3.fill = java.awt.GridBagConstraints.BOTH;
        panelRuleList.add(panelRuleListSelect, gridBagConstraints3);
        
        
        scrollPanelRuleList.setPreferredSize(new java.awt.Dimension(380, 230));
        scrollPanelRuleList.setMinimumSize(new java.awt.Dimension(380, 180));
        
        tableRuleList.setModel(new javax.swing.table.DefaultTableModel (
        new Object [][] {
            
        },
        new String [] {
            "State", "Read", "Write", "Move", "Next State"
        }
        ) {
            Class[] types = new Class [] {
                java.lang.Integer.class, java.lang.Character.class, java.lang.Character.class, java.lang.Character.class, java.lang.Integer.class
            };
            
            public Class getColumnClass (int columnIndex) {
                return types [columnIndex];
            }
        });
        tableRuleList.setMinimumSize(new java.awt.Dimension(280, 180));
        scrollPanelRuleList.setViewportView(tableRuleList);
        
        gridBagConstraints3 = new java.awt.GridBagConstraints();
        gridBagConstraints3.gridx = 0;
        gridBagConstraints3.gridwidth = 12;
        gridBagConstraints3.gridheight = 8;
        gridBagConstraints3.fill = java.awt.GridBagConstraints.BOTH;
        panelRuleList.add(scrollPanelRuleList, gridBagConstraints3);
        
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 6;
        gridBagConstraints1.gridwidth = 16;
        gridBagConstraints1.gridheight = 10;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
        getContentPane().add(panelRuleList, gridBagConstraints1);
        
        
        panelInput.setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gridBagConstraints5;
        panelInput.setBorder(new javax.swing.border.TitledBorder("Input/Config"));
        panelInput.setPreferredSize(new java.awt.Dimension(200, 100));
        panelInput.setMinimumSize(new java.awt.Dimension(200, 100));
        
        textFieldInput.setText(Fields.input);
        textFieldInput.setPreferredSize(new java.awt.Dimension(200, 23));
        textFieldInput.setMaximumSize(new java.awt.Dimension(200, 23));
        textFieldInput.setMinimumSize(new java.awt.Dimension(100, 23));
        gridBagConstraints5 = new java.awt.GridBagConstraints();
        gridBagConstraints5.gridx = 0;
        gridBagConstraints5.gridy = 0;
        gridBagConstraints5.gridwidth = 4;
        gridBagConstraints5.gridheight = 2;
        panelInput.add(textFieldInput, gridBagConstraints5);
        
        
        buttonInputLoad.setText("Load");
        buttonInputLoad.setPreferredSize(new java.awt.Dimension(75, 25));
        buttonInputLoad.setMinimumSize(new java.awt.Dimension(75, 25));
        buttonInputLoad.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                buttonInputLoadActionPerformed(evt);
            }
        }
        );
        gridBagConstraints5 = new java.awt.GridBagConstraints();
        gridBagConstraints5.gridx = 4;
        gridBagConstraints5.gridy = 0;
        gridBagConstraints5.gridwidth = 4;
        gridBagConstraints5.gridheight = 2;
        panelInput.add(buttonInputLoad, gridBagConstraints5);
        
        
        labelStateInitial.setText("Initial State:");
        gridBagConstraints5 = new java.awt.GridBagConstraints();
        gridBagConstraints5.gridx = 0;
        gridBagConstraints5.gridy = 2;
        gridBagConstraints5.gridwidth = 4;
        panelInput.add(labelStateInitial, gridBagConstraints5);
        
        
        textFieldStateInitial.setText("" + Fields.initialState);
        textFieldStateInitial.setPreferredSize(new java.awt.Dimension(61, 23));
        textFieldStateInitial.setMinimumSize(new java.awt.Dimension(61, 23));
        textFieldStateInitial.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                textFieldStateInitialActionPerformed(evt);
            }
        }
        );
        gridBagConstraints5 = new java.awt.GridBagConstraints();
        gridBagConstraints5.gridx = 4;
        gridBagConstraints5.gridy = 2;
        gridBagConstraints5.gridwidth = 4;
        panelInput.add(textFieldStateInitial, gridBagConstraints5);
        
        
        labelStateFinal.setText("Final State:");
        gridBagConstraints5 = new java.awt.GridBagConstraints();
        gridBagConstraints5.gridx = 0;
        gridBagConstraints5.gridy = 3;
        gridBagConstraints5.gridwidth = 4;
        panelInput.add(labelStateFinal, gridBagConstraints5);
        
        
        textFieldStateFinal.setText("" + Fields.finalState);
        textFieldStateFinal.setPreferredSize(new java.awt.Dimension(61, 23));
        textFieldStateFinal.setMinimumSize(new java.awt.Dimension(61, 23));
        textFieldStateFinal.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                textFieldStateFinalActionPerformed(evt);
            }
        }
        );
        gridBagConstraints5 = new java.awt.GridBagConstraints();
        gridBagConstraints5.gridx = 4;
        gridBagConstraints5.gridy = 3;
        gridBagConstraints5.gridwidth = 4;
        panelInput.add(textFieldStateFinal, gridBagConstraints5);
        
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 16;
        gridBagConstraints1.gridy = 4;
        gridBagConstraints1.gridwidth = 8;
        gridBagConstraints1.gridheight = 4;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
        getContentPane().add(panelInput, gridBagConstraints1);
        
        
        panelOptions.setLayout(new javax.swing.BoxLayout(panelOptions, 1));
        panelOptions.setBorder(new javax.swing.border.TitledBorder("Options"));
        panelOptions.setPreferredSize(new java.awt.Dimension(200, 150));
        panelOptions.setMinimumSize(new java.awt.Dimension(200, 150));
        
        panelSpeed.setLayout(new javax.swing.BoxLayout(panelSpeed, 1));
        panelSpeed.setPreferredSize(new java.awt.Dimension(150, 75));
        panelSpeed.setMinimumSize(new java.awt.Dimension(150, 75));
        
        labelSpeed.setText("Speed");
        panelSpeed.add(labelSpeed);
        
        
        sliderSpeed.setMinorTickSpacing(10);
        sliderSpeed.setPaintLabels(true);
        sliderSpeed.setPaintTicks(true);
        sliderSpeed.setMajorTickSpacing(20);
        sliderSpeed.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                sliderSpeedStateChanged(evt);
            }
        }
        );
        panelSpeed.add(sliderSpeed);
        
        panelOptions.add(panelSpeed);
        
        
        panelShowRule.setPreferredSize(new java.awt.Dimension(200, 40));
        panelShowRule.setMinimumSize(new java.awt.Dimension(200, 40));
        
        checkBoxShowCurrentRule.setForeground(new java.awt.Color (102, 102, 153));
        checkBoxShowCurrentRule.setText("Show current rule");
        checkBoxShowCurrentRule.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                checkBoxShowCurrentRuleItemStateChanged(evt);
            }
        }
        );
        panelShowRule.add(checkBoxShowCurrentRule);
        
        panelOptions.add(panelShowRule);
        
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 16;
        gridBagConstraints1.gridy = 8;
        gridBagConstraints1.gridwidth = 8;
        gridBagConstraints1.gridheight = 6;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
        getContentPane().add(panelOptions, gridBagConstraints1);
        
        
        panelStats.setLayout(new javax.swing.BoxLayout(panelStats, 1));
        panelStats.setBorder(new javax.swing.border.TitledBorder("Statistics"));
        panelStats.setPreferredSize(new java.awt.Dimension(200, 50));
        panelStats.setMinimumSize(new java.awt.Dimension(200, 50));
        
        labelStatsSteps.setText("Steps " + Fields.stepCounter);
        labelStatsSteps.setPreferredSize(new java.awt.Dimension(200, 50));
        labelStatsSteps.setMinimumSize(new java.awt.Dimension(190, 40));
        labelStatsSteps.setMaximumSize(new java.awt.Dimension(300, 50));
        panelStats.add(labelStatsSteps);
        
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 16;
        gridBagConstraints1.gridy = 14;
        gridBagConstraints1.gridwidth = 8;
        gridBagConstraints1.gridheight = 2;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
        getContentPane().add(panelStats, gridBagConstraints1);
        
        
        panelMessages.setLayout(new java.awt.GridBagLayout());
        java.awt.GridBagConstraints gridBagConstraints6;
        panelMessages.setBorder(new javax.swing.border.TitledBorder("Messages"));
        panelMessages.setPreferredSize(new java.awt.Dimension(600, 100));
        panelMessages.setMinimumSize(new java.awt.Dimension(600, 100));
        
        scrollPaneMessages.setPreferredSize(new java.awt.Dimension(580, 75));
        scrollPaneMessages.setMinimumSize(new java.awt.Dimension(580, 75));
        scrollPaneMessages.setDoubleBuffered(true);
        
        textAreaMessages.setEditable(false);
        textAreaMessages.setMargin(new java.awt.Insets(5, 5, 5, 5));
        textAreaMessages.setMinimumSize(new java.awt.Dimension(560, 50));
        scrollPaneMessages.setViewportView(textAreaMessages);
        
        gridBagConstraints6 = new java.awt.GridBagConstraints();
        gridBagConstraints6.gridx = 0;
        gridBagConstraints6.gridwidth = 24;
        gridBagConstraints6.gridheight = 4;
        gridBagConstraints6.fill = java.awt.GridBagConstraints.BOTH;
        panelMessages.add(scrollPaneMessages, gridBagConstraints6);
        
        
        gridBagConstraints1 = new java.awt.GridBagConstraints();
        gridBagConstraints1.gridx = 0;
        gridBagConstraints1.gridy = 16;
        gridBagConstraints1.gridwidth = 24;
        gridBagConstraints1.gridheight = 4;
        gridBagConstraints1.fill = java.awt.GridBagConstraints.BOTH;
        getContentPane().add(panelMessages, gridBagConstraints1);
        
    }//GEN-END:initComponents

    private void comboBoxRuleListActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_comboBoxRuleListActionPerformed
        JComboBox jcb = (JComboBox) evt.getSource();
        String st = (String) jcb.getSelectedItem();
        int index = jcb.getSelectedIndex();
        System.out.println("action " + st + index);
    }//GEN-LAST:event_comboBoxRuleListActionPerformed

    private void checkBoxShowCurrentRuleItemStateChanged(java.awt.event.ItemEvent evt) {//GEN-FIRST:event_checkBoxShowCurrentRuleItemStateChanged
        JCheckBox jcb = (JCheckBox) evt.getSource();
        if (Fields.showCurrentRule) {
            Fields.showCurrentRule = false;
            showMessage("Showing current rule disabled!");
        } else {
            Fields.showCurrentRule = true;
            showMessage("Showing current rule enabled!");
        }
    }//GEN-LAST:event_checkBoxShowCurrentRuleItemStateChanged

    private void textFieldStateFinalActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_textFieldStateFinalActionPerformed
        Fields.finalState = Integer.parseInt(textFieldStateFinal.getText());
    }//GEN-LAST:event_textFieldStateFinalActionPerformed

    private void comboBoxRuleListItemStateChanged(java.awt.event.ItemEvent evt) {//GEN-FIRST:event_comboBoxRuleListItemStateChanged
        JComboBox jcb = (JComboBox) evt.getSource();
        String st = (String) jcb.getSelectedItem();
        Fields.selection = jcb.getSelectedIndex();
        System.out.println("action " + st + Fields.selection);
    }//GEN-LAST:event_comboBoxRuleListItemStateChanged

    private void textFieldStateInitialActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_textFieldStateInitialActionPerformed
        Fields.initialState = Integer.parseInt(textFieldStateInitial.getText());
    }//GEN-LAST:event_textFieldStateInitialActionPerformed

    private void buttonStepActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_buttonStepActionPerformed
        switch (Fields.tmRunMode) {
            case 0:
                if (Fields.ruleList[Fields.selection] == null) {
                    showMessage("ERROR: Could not find rule list!");
                    showMessage("Please load or create rule list!");
                } else {
                    Fields.tmRunMode = 4;
                    doSingleStep();
                    showMessage("Next step.");
                }
                break;
            case 1: 
                showMessage("Cant't go to step mode while Turing Machine is running!");
                showMessage("Please pause or reset Turing Machine first!");
                break;
            case 2:
            case 4:
                Fields.tmRunMode = 4;
                doSingleStep();
                showMessage("Next step.");
                break;
            case 3:
                showMessage("Turing Machine is stopped!");
                showMessage("Please reset Turing Machine first!");
                break;
            default:     
                System.out.println("buttonStepActionPerformed");             
                break;  
        }        
    }//GEN-LAST:event_buttonStepActionPerformed

    private void buttonInputLoadActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_buttonInputLoadActionPerformed
        Fields.input = textFieldInput.getText();
        initTape();
        panelTuringMachine.repaint();
        showMessage("Input set to " + Fields.input);
    }//GEN-LAST:event_buttonInputLoadActionPerformed

    /* sets the new speed value whenever the slider changes its state*/
    private void sliderSpeedStateChanged(javax.swing.event.ChangeEvent evt) {//GEN-FIRST:event_sliderSpeedStateChanged
        Fields.speed = sliderSpeed.getValue();
        if (Fields.timer != null) {
            Fields.timer.setDelay(Fields.maxWaitingTime / (1 + sliderSpeed.getValue()));
        }
        System.out.println("Speed set to " + Fields.speed);
        showMessage("Speed set to " + Fields.speed);
    }//GEN-LAST:event_sliderSpeedStateChanged

    /* the reset button was pressed */
    private void buttonResetActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_buttonResetActionPerformed
        switch (Fields.tmRunMode) {
            case 0:
                break;
            case 1:
            case 2:
            case 3:
            case 4:        
                /* stop the turing machine */
                Fields.tmRunMode = 0;
                if (Fields.timer != null) {
                    Fields.timer.stop();
                    Fields.timer = null;
                }
                /* reset configuration */
                initTape();
                labelStatsSteps.setText("Steps: " + (Fields.stepCounter = 0));
                Fields.initialState = Integer.parseInt(textFieldStateInitial.getText());
                Fields.finalState = Integer.parseInt(textFieldStateFinal.getText());
                Fields.currentState = Fields.initialState;
                panelTuringMachine.repaint();
                showMessage("Turing Machine resetted!");
                break;
            default:
                System.out.println("buttonResetActionPerformed");
        }
    }//GEN-LAST:event_buttonResetActionPerformed

    /* the pause button was pressed */
    private void buttonPauseActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_buttonPauseActionPerformed
        switch (Fields.tmRunMode) {
            case 0:
                break;
            case 1:
                Fields.tmRunMode = 2;
                if (Fields.timer.isRunning()) {
                    Fields.timer.stop();
                }
                break;
            case 2:
            case 3:
            case 4:
                break;
            default:
                System.out.println("buttonPauseActionPerformed");
        }
    }//GEN-LAST:event_buttonPauseActionPerformed

    private void buttonStartActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_buttonStartActionPerformed
        switch (Fields.tmRunMode) {
            case 0:
                if (Fields.selection == -1) {
                    showMessage("ERROR: No rule list specified!");
                    showMessage("Please load or create rule list!");
                }
                Fields.tmRunMode = 1;
                Fields.initialState = Integer.parseInt(textFieldStateInitial.getText());
                Fields.finalState = Integer.parseInt(textFieldStateFinal.getText());
                Fields.timer = new javax.swing.Timer(Fields.maxWaitingTime / (1 + Fields.speed),
                    new ActionListener() {
                        public void actionPerformed(ActionEvent evt) {
                            doSingleStep();
                        }    
                    }
                );
                Fields.timer.start();
                break;
            case 1:
                break;
            case 2:
                Fields.tmRunMode = 1;
                Fields.timer.restart();
                break;
            case 3:
                showMessage("Turing Machine is stopped!");
                showMessage("Please reset Turing Machine first!");
                break;
            case 4:
                Fields.tmRunMode = 1;
                Fields.timer = new javax.swing.Timer(Fields.maxWaitingTime / (1 + Fields.speed),
                    new ActionListener() {
                        public void actionPerformed(ActionEvent evt) {
                            doSingleStep();
                        }    
                    }
                );
                Fields.timer.start();
                break;
            default:
                break;
        }
    }//GEN-LAST:event_buttonStartActionPerformed

    private void buttonRuleListLoadActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_buttonRuleListLoadActionPerformed
	Fields.selection = comboBoxRuleList.getSelectedIndex();
	if (Fields.ruleListNames[Fields.selection] == "file") {
	    JFileChooser chooser = new JFileChooser(Fields.currentDirectory);
	    if (chooser.showOpenDialog(null) == JFileChooser.APPROVE_OPTION) {
		Fields.selectedFile = chooser.getSelectedFile().getPath();
		Fields.currentDirectory = chooser.getSelectedFile().getParent();	
		showMessage("Selected file " + Fields.selectedFile + ".");
		readRuleLists();
	    }
	}
        createTable();
	showMessage("Rule list " + Fields.ruleListNames[Fields.selection] + " successfully loaded.");
    }//GEN-LAST:event_buttonRuleListLoadActionPerformed


    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JMenuBar MenuBar;
    private javax.swing.JMenu File;
    private javax.swing.JMenu Options;
    private javax.swing.JPanel panelTuringMachine;
    private javax.swing.JPanel panelControl;
    private javax.swing.JButton buttonStart;
    private javax.swing.JButton buttonPause;
    private javax.swing.JButton buttonReset;
    private javax.swing.JButton buttonStep;
    private javax.swing.JPanel panelRuleList;
    private javax.swing.JPanel panelRuleListSelect;
    private javax.swing.JComboBox comboBoxRuleList;
    private javax.swing.JButton buttonRuleListLoad;
    private javax.swing.JScrollPane scrollPanelRuleList;
    private javax.swing.JTable tableRuleList;
    private javax.swing.JPanel panelInput;
    private javax.swing.JTextField textFieldInput;
    private javax.swing.JButton buttonInputLoad;
    private javax.swing.JLabel labelStateInitial;
    private javax.swing.JTextField textFieldStateInitial;
    private javax.swing.JLabel labelStateFinal;
    private javax.swing.JTextField textFieldStateFinal;
    private javax.swing.JPanel panelOptions;
    private javax.swing.JPanel panelSpeed;
    private javax.swing.JLabel labelSpeed;
    private javax.swing.JSlider sliderSpeed;
    private javax.swing.JPanel panelShowRule;
    private javax.swing.JCheckBox checkBoxShowCurrentRule;
    private javax.swing.JPanel panelStats;
    private javax.swing.JLabel labelStatsSteps;
    private javax.swing.JPanel panelMessages;
    private javax.swing.JScrollPane scrollPaneMessages;
    private javax.swing.JTextArea textAreaMessages;
    // End of variables declaration//GEN-END:variables
}